/***************************************************************************
 *   Copyright (C) 2004 by Mauricio Oliveira Carneiro (PROCC - FIOCRUZ)    *
 *   carneiro@fiocruz.br                                                   *
 *   http://www.procc.fiocruz.br/                                          * 
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include "evolmodel.h"

void evolutionarymodel_init_JC(EvolutionaryModel *model)
{
	int i,j;
	for (i = 0; i < 4; i++) {
		for (j = 0; j < 4; j++) {
			if (i != j) (*model)[i][j] = 0.33;
			else (*model)[i][j] = 0;
		}
	}
}

void evolutionarymodel_init_K2P(EvolutionaryModel *model, double transition, double transversion)
{
	/* A */
	(*model)[0][0] = 0.0; 
	(*model)[1][0] = transversion; 
	(*model)[2][0] = transversion; 
	(*model)[3][0] = transition;
	
	/* C */
	(*model)[0][1] = transversion; 
	(*model)[1][1] = 0.0; 
	(*model)[2][1] = transition; 
	(*model)[3][1] = transversion;
	
	/* G */
	(*model)[0][2] = transversion; 
	(*model)[1][2] = transition; 
	(*model)[2][2] = 0.0; 
	(*model)[3][2] = transversion;
	
	/* T */
	(*model)[0][3] = transition; 
	(*model)[1][3] = transversion; 
	(*model)[2][3] = transversion; 
	(*model)[3][3] = 0.0;
}

void evolutionarymodel_init_REV(EvolutionaryModel *model, double AC, double AG, double AT, double CG, double CT, double GT)
{
	/* A */
	(*model)[0][0] = 0.0; 
	(*model)[1][0] = AC; 
	(*model)[2][0] = AG; 
	(*model)[3][0] = AT;
	
	/* C */
	(*model)[0][1] = AC; 
	(*model)[1][1] = 0.0; 
	(*model)[2][1] = CG; 
	(*model)[3][1] = CT;
	
	/* G */
	(*model)[0][2] = AG; 
	(*model)[1][2] = CG; 
	(*model)[2][2] = 0.0; 
	(*model)[3][2] = GT;
	
	/* T */
	(*model)[0][3] = AT; 
	(*model)[1][3] = CT; 
	(*model)[2][3] = GT; 
	(*model)[3][3] = 0.0;
}



char apply_model (EvolutionaryModel model, char from, int to) 
{
	int old_char;
	int sum;
	switch (from) {
		case 'A': old_char = A; break;
		case 'C': old_char = C; break;
		case 'G': old_char = G; break;
		case 'T': old_char = T; break;
	}
	sum = 0;
	if (to < sum + (int) (model[old_char][A] * 100)) return 'A';
	sum += (int) (model[old_char][A] * 100);
	if (to < sum + (int) (model[old_char][C] * 100)) return 'C';
	sum += (int) (model[old_char][C] * 100);
	if (to < sum + (int) (model[old_char][G] * 100)) return 'G';
	return 'T';
}
