/***************************************************************************
 *   Copyright (C) 2004 by Mauricio Oliveira Carneiro (PROCC - FIOCRUZ)    *
 *   carneiro@fiocruz.br                                                   *
 *   http://www.procc.fiocruz.br/                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __STRAIN_H
#define __STRAIN_H

#include "evolmodel.h"
#include "host.h"

typedef struct nucleotide {
	char Value;
} Sequence, *pSequence;

typedef struct strain {
	pSequence Nucleotides;
	int nNucleotides;
	char Name[50];
	int nMutationsPerEvolution;
	int iGeneration;
	int iSeqCount;  /* Responsible for the uniqueness of the name */
} Strain, *pStrain;

pStrain strain_init (int nNucleotides, int nMutationsPerEvolution, char *sName);
void strain_generate_random (pStrain s, double rateA, double rateC, double rateG, double rateT);
void strain_read_fasta (pStrain s, char *filename);
void strain_show (pStrain s);
void strain_output_fasta (pStrain s);
void strain_delete (pStrain *s);
void strain_evolve (pHost h, EvolutionaryModel Model, pStrain s, int nGenerations, int iGeneration);

#endif
